/* Copyright 1996 Free Software Foundation, Inc.
   Contributed by Marcin Dalecki <dalecki@sub994.sub.uni-goettingen.de>

   This file is part of the Linux modutils.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

#ifndef MISC_H
#define MISC_H

#include <stdio.h>

#define LINECAT_CHAR '\\'
#define COMMENT_CHAR '#'

#include "module.h"

#define kernel_sym  old_kernel_sym

extern void load_kernel_symbols (void);
extern struct kernel_sym *ksymtab;
extern int nksyms;

enum SYM_STATUS
  {
    SYM_PASUTIL,		/* required by some other module */
    SYM_DEFINED,		/* found in the current module */
    SYM_REQUIRED		/* required by some module */
  };

/*
 * Globally used data definitions.
 */

struct MODULE;

struct SYMBOL
  {
    struct SYMBOL *next;
    struct MODULE *module;	/* module this symbol is declared in */
    char *name;
    char req;
    char def;
  };

extern struct SYMBOL *syms_hash[2048];

extern struct SYMBOL *add_sym (const char *, struct MODULE *,
			       enum SYM_STATUS, int *);

struct MODULE
  {
    struct MODULE *next;	/* link into the global list */
    char *name;
    struct
      {				/* public and external symbols */
	struct SYMBOL **tb;
	int nb;
      }
    pub, ext;
  };

extern int debugmode;

extern int load_obj (const char *modname, struct MODULE *mod);

/*
 * String manipulation utilities.
 */
extern const char *strip_o (const char *fname);

/*
 * File manipulation
 */

extern char *read_and_preprocess_file (const char *);
char *get_concat_line (char *from, int *lines);

/*
 * Error logging facilities.
 */

extern int errors;
extern const char *error_file;
extern int log;

void
error (const char *fmt,...)
#ifdef __GNUC__
__attribute__ ((format (printf, 1, 2)))
#endif
;

     void lprintf (const char *fmt,...)
#ifdef __GNUC__
__attribute__ ((format (printf, 1, 2)))
#endif
;

     extern void setsyslog (const char *program);

#endif /* MISC_H */
