/* Copyright 1997 Free Software Foundation, Inc.
   Contributed by Marcin Dalecki <dalecki@sub994.sub.uni-goettingen.de>

   This file is part of the Linux modutils.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

#include <stdio.h>
#include <string.h>

#include "util.h"
#include "misc.h"

/*
 * Add a symbol to the symbol list of module.
 * Returns the pointer to it on success.
 */
struct SYMBOL *
add_sym (const char *name, struct MODULE *module, enum SYM_STATUS status,
	 int *module_requis)
{
  unsigned hashval = 0;
  const char *pt = name;
  struct SYMBOL **ffound;
  struct SYMBOL *found;

  while (*pt != '\0')
    {
      hashval = (hashval << 1) + *pt++;
    }
  hashval %= 2048;
  ffound = syms_hash + hashval;
  found = *ffound;
  while (found != NULL)
    {
      if (strcmp (found->name, name) == 0)
	break;
      found = found->next;
    }
  if (found == NULL)
    {
      /*
       * Add it to the list
       */
      found = (struct SYMBOL *) xmalloc (sizeof (struct SYMBOL));
      found->name = xstrdup (name);
      found->module = NULL;
      found->next = *ffound;
      *ffound = found;
      if (status == SYM_REQUIRED)
	{
	  found->req = 1;
	  found->def = 0;
	}
      else if (status == SYM_PASUTIL)
	{
	  found->req = 0;
	  found->def = 0;
	}
      else
	{
	  found->req = 0;
	  found->def = 1;
	  found->module = module;
	}
    }
  else
    {
      if (status == SYM_DEFINED)
	{
	  /*
	   * This symbol is defined in the current module; add it to the list.
	   */
	  if (found->def)
	    {
	      found = (struct SYMBOL *) xmalloc (sizeof (struct SYMBOL));
	      found->name = xstrdup (name);
	      found->next = NULL;
	      found->req = 0;
	    }
	  found->module = module;
	  found->def = 1;
	  if (found->req)
	    *module_requis = 1;
	}
      else if (status == SYM_REQUIRED)
	found->req = 1;
    }

  return found;
}
